@@ -249,33 +249,50 @@ def runProbSolver(p_, solver_str_or_instance=None, *args, **kwargs):
 
     
     try:
-        if 0 and isConverter:
-            pass
-            # TODO: will R be somewhere used?
-            #R = converter(solverName, **solver_params)
-        else:
-            nErr = check(p)
-            if nErr: p.err("prob check results: " +str(nErr) + "ERRORS!")#however, I guess this line will be never reached.
-            if p.probType not in ('IP', 'EIG'): p.iterfcn(p.x0)
-            if hasSetproctitleModule:
-                try:
-                    originalName = setproctitle.getproctitle()
-                    if originalName.startswith('OpenOpt-'):
-                        originalName = None
-                    else:
-                        s = 'OpenOpt-' + p.solver.__name__
-                        # if p.name != 'unnamed':
-                        s += '-' + p.name
-                        setproctitle.setproctitle(s)
-                except:
+#        if 0 and isConverter:
+#            pass
+#            # TODO: will R be somewhere used?
+#            #R = converter(solverName, **solver_params)
+#        else:
+        nErr = check(p)
+        if nErr: p.err("prob check results: " +str(nErr) + "ERRORS!")#however, I guess this line will be never reached.
+        if p.probType not in ('IP', 'EIG'): p.iterfcn(p.x0)
+        if hasSetproctitleModule:
+            try:
+                originalName = setproctitle.getproctitle()
+                if originalName.startswith('OpenOpt-'):
                     originalName = None
-            else:
-                p.pWarn('''
-                please install setproctitle module 
-                (it's available via easy_install and Linux soft channels like apt-get)''')
-            solver(p)
-            if hasSetproctitleModule and originalName is not None:
-                setproctitle.setproctitle(originalName)
+                else:
+                    s = 'OpenOpt-' + p.solver.__name__
+                    # if p.name != 'unnamed':
+                    s += '-' + p.name
+                    setproctitle.setproctitle(s)
+            except:
+                originalName = None
+        else:
+            p.pWarn('''
+            please install setproctitle module 
+            (it's available via easy_install and Linux soft channels like apt-get)''')
+        
+#        import os, cProfile
+#        print '----'
+#        print 'p.n, p.nb, p.nbeq, p.nc, p.nh:',  p.n, p.nb, p.nbeq, p.nc, p.nh
+#        if p.solver.__name__ == 'interalg':
+#            cProfile.runctx( 'solver(p)', globals(), locals(), filename="t.profile" )
+#        #os.system('runsnake t.profile')
+#        else:
+#            solver(p)
+            
+        solver(p)
+        
+#        if p.solver.__name__ == 'interalg':
+#            
+#            print p.extras
+#            os.system('pyprof2calltree -i t.profile -o myfile.prof.grind')
+#            os.system('kcachegrind myfile.prof.grind')
+#        print p.xk
+#        print '===='
+
 #    except killThread:
 #        if p.plot:
 #            print 'exiting pylab'
@@ -297,12 +314,19 @@ def runProbSolver(p_, solver_str_or_instance=None, *args, **kwargs):
         if p.istop == 0: p.istop = 1000
     finally:
         seterr(**old_err)
+        
+    if hasSetproctitleModule and originalName is not None:
+        setproctitle.setproctitle(originalName)
+    
     ############################
     p.contol = p.primalConTol
 
     # Solving finished
     if hasattr(p, '_bestPoint') and not any(isnan(p._bestPoint.x)) and p.probType != 'ODE':
-        p.iterfcn(p._bestPoint)
+        try:
+            p.iterfcn(p._bestPoint)
+        except isSolved:
+            pass
     if p.probType != 'EIG':
         if not hasattr(p, 'xf') and not hasattr(p, 'xk'): p.xf = p.xk = ones(p.n)*nan
         if hasattr(p, 'xf') and (not hasattr(p, 'xk') or array_equal(p.xk, p.x0)): p.xk = p.xf
@@ -342,7 +366,7 @@ def runProbSolver(p_, solver_str_or_instance=None, *args, **kwargs):
     #if not hasattr(p, 'xf'): p.xf = p.xk
     if type(p.xf) in (list, tuple) or isscalar(p.xf): p.xf = asarray(p.xf)
     p.xf = p.xf.flatten()
-    p.rf = p.getMaxResidual(p.xf) if not p.probType == 'IP' else p.rk
+    p.rf = p.getMaxResidual(p.xf) if p.probType not in ('IP', 'ODE') else p.rk
 
     if not p.isFeasible and p.istop > 0: p.istop = -100-p.istop/1000.0
     
@@ -380,6 +404,9 @@ def runProbSolver(p_, solver_str_or_instance=None, *args, **kwargs):
 
 ##################################################################
 def finalTextOutput(p, r):
+    if type(p.msg) == ndarray: # from fmincon etc
+        msg = asarray(p.msg.flatten(),int).tolist()
+        p.msg = ''.join(chr(c) for c in msg)
     if p.iprint >= 0:
         if len(p.msg):  
             p.disp("istop: " + str(r.istop) + ' (' + p.msg +')')
